class IdentityKeyStore:
    def init(self, state):
        self.state = state  # instance of x3dh.State

    def get_identity_key_pair(self):
        return self.state._BaseState__identity_key

    def get_signed_prekey(self):
        return self.state._BaseState__signed_pre_key

    def get_one_time_prekey(self):
        try:
            return next(iter(self.state._BaseState__pre_keys)).pub
        except StopIteration:
            raise Exception("No available one-time prekeys.")

    def remove_used_one_time_prekey(self, pub_key: bytes):
        self.state.delete_pre_key(pub_key)



import pickle
import os

class SessionStore:
    def init(self, path='sessions'):
        self.path = path
        os.makedirs(path, exist_ok=True)

    def _session_file(self, recipient_id: str) -> str:
        return os.path.join(self.path, f"{recipient_id}.pkl")

    def store_session(self, recipient_id: str, session):
        with open(self._session_file(recipient_id), 'wb') as f:
            pickle.dump(session, f)

    def load_session(self, recipient_id: str):
        try:
            with open(self._session_file(recipient_id), 'rb') as f:
                return pickle.load(f)
        except FileNotFoundError:
            return None



from message_pb2 import MessageEnvelope

def create_envelope(header, ciphertext):
    envelope = MessageEnvelope()
    envelope.sender_identity_key = header.identity_key
    envelope.sender_ephemeral_key = header.ephemeral_key
    envelope.sender_signed_prekey = header.signed_pre_key
    if header.pre_key:
        envelope.one_time_prekey = header.pre_key
    envelope.ciphertext = ciphertext
    return envelope.SerializeToString()

def parse_envelope(serialized_msg):
    envelope = MessageEnvelope()
    envelope.ParseFromString(serialized_msg)
    return envelope
